/* 
 * GTKSEISVIEWGL - Library for rendering of 2D seismic data
 *
 * Copyright (C) 2006 Vladimir Bashkardin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more av.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author:  Vladimir Bashkardin  <vovizmus@users.sourceforge.net>
 */

#ifndef __G_SEIS_VIEW_TOOL_H__
#define __G_SEIS_VIEW_TOOL_H__

#include <gtk/gtk.h>
#include <gtkseisviewgl/gseisviewdatavis.h>

G_BEGIN_DECLS

#define G_SEIS_TYPE_VIEW_TOOL            (g_seis_view_tool_get_type ())
#define G_SEIS_VIEW_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_SEIS_TYPE_VIEW_TOOL, GSeisViewTool))
#define G_SEIS_VIEW_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), G_SEIS_TYPE_VIEW_TOOL, GSeisViewToolClass))
#define G_SEIS_IS_VIEW_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_SEIS_TYPE_VIEW_TOOL))
#define G_SEIS_IS_VIEW_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_SEIS_TYPE_VIEW_TOOL))
#define G_SEIS_VIEW_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), G_SEIS_TYPE_VIEW_TOOL, GSeisViewToolClass))

typedef struct _GSeisViewTool GSeisViewTool;
typedef struct _GSeisViewToolClass GSeisViewToolClass;

typedef enum {
  G_SEIS_VIEW_TOOL_ENTER_EVENT	= 1 << 1,
  G_SEIS_VIEW_TOOL_LEAVE_EVENT	= 1 << 2,
  G_SEIS_VIEW_TOOL_MOTION_EVENT	= 1 << 3,
  G_SEIS_VIEW_TOOL_BUTTON_EVENT	= 1 << 4,
  G_SEIS_VIEW_TOOL_SCROLL_EVENT	= 1 << 5,
  G_SEIS_VIEW_TOOL_KEY_EVENT	= 1 << 6,
  G_SEIS_VIEW_TOOL_REDRAW_EVENT	= 1 << 7,
  G_SEIS_VIEW_TOOL_RESIZE_EVENT	= 1 << 8,
  G_SEIS_VIEW_TOOL_ALL_EVENTS	= (1 << 9) - 1
} GSeisViewToolEventMask;

struct _GSeisViewTool {
    GObject parent;

    GSeisViewToolEventMask event_mask;

    gboolean show_cross;
};

struct _GSeisViewToolClass {
    GObjectClass parent_class;

	void (*view_attach_notify) (GSeisViewTool *self, GtkWidget *seis_view_gl);
	void (*view_detach_notify) (GSeisViewTool *self, GtkWidget *seis_view_gl);

	void (*view_button_event) (GSeisViewTool *self, GtkWidget *seis_view_gl, GdkEventButton *event,
                               GSeisViewDataVisibility *data_visibility);
	void (*view_scroll_event) (GSeisViewTool *self, GtkWidget *seis_view_gl, GdkEventScroll *event,
                               GSeisViewDataVisibility *data_visibility);
	void (*view_enter_event) (GSeisViewTool *self, GtkWidget *seis_view_gl, GdkEventCrossing *event,
                              GSeisViewDataVisibility *data_visibility);
	void (*view_leave_event) (GSeisViewTool *self, GtkWidget *seis_view_gl, GdkEventCrossing *event,
                              GSeisViewDataVisibility *data_visibility);
	void (*view_motion_event) (GSeisViewTool *self, GtkWidget *seis_view_gl, GdkEventMotion *event,
                               GSeisViewDataVisibility *data_visibility);
	void (*view_key_event) (GSeisViewTool *self, GtkWidget *seis_view_gl, GdkEventKey *event,
                            GSeisViewDataVisibility *data_visibility);

	void (*view_redraw_event) (GSeisViewTool *self, GtkWidget *seis_view_gl,
                               GSeisViewDataVisibility *data_visibility);
	void (*view_resize_event) (GSeisViewTool *self, GtkWidget *seis_view_gl,
                               GSeisViewDataVisibility *data_visibility);
};

GType g_seis_view_tool_get_type (void);

void g_seis_view_tool_set_alt_tool1 (GSeisViewTool *tool, GSeisViewTool *alt_tool1);
void g_seis_view_tool_set_alt_tool2 (GSeisViewTool *tool, GSeisViewTool *alt_tool2);
void g_seis_view_tool_set_alt_tool3 (GSeisViewTool *tool, GSeisViewTool *alt_tool2);

void g_seis_view_tool_attach_notify (GSeisViewTool *tool, GtkWidget *seis_view_gl);
void g_seis_view_tool_detach_notify (GSeisViewTool *tool, GtkWidget *seis_view_gl);
void g_seis_view_tool_button_event (GSeisViewTool *tool, GtkWidget *seis_view_gl, GdkEventButton *event,
                                    GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_scroll_event (GSeisViewTool *tool, GtkWidget *seis_view_gl, GdkEventScroll *event,
                                    GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_enter_event (GSeisViewTool *tool, GtkWidget *seis_view_gl, GdkEventCrossing *event,
                                   GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_leave_event (GSeisViewTool *tool, GtkWidget *seis_view_gl, GdkEventCrossing *event,
                                   GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_motion_event (GSeisViewTool *tool, GtkWidget *seis_view_gl, GdkEventMotion *event,
                                    GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_key_event (GSeisViewTool *tool, GtkWidget *seis_view_gl, GdkEventKey *event,
                                 GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_redraw_event (GSeisViewTool *tool, GtkWidget *seis_view_gl,
                                    GSeisViewDataVisibility *data_visibility);
void g_seis_view_tool_resize_event (GSeisViewTool *tool, GtkWidget *seis_view_gl,
                                    GSeisViewDataVisibility *data_visibility);

G_END_DECLS

#endif /* __G_SEIS_VIEW_TOOL_H__ */
